@@ -22,10 +22,12 @@ from loguru import logger
 from data_diff.abcs.database_types import (
     JSON,
     Boolean,
+    ColType,
     ColType_UUID,
     Date,
     Datetime,
     DbPath,
+    DbTime,
     Decimal,
     Float,
     FractionalType,
@@ -49,6 +51,7 @@ from data_diff.databases.base import (
     ThreadedDatabase,
     import_helper,
 )
+from data_diff.schema import RawColumnInfo
 
 
 @import_helper("sybase")
@@ -196,9 +199,25 @@ class Dialect(BaseDialect):
         return f"VALUES {values}"
 
     def normalize_timestamp(self, value: str, coltype: TemporalType) -> str:
+        varchar_type = (
+            "VARCHAR"
+            if (self.sybase_driver_type.is_iq or self.query_config_for_free_tds["freetds_query_chosen"])
+            else "NVARCHAR"
+        )
+
+        # Handle Date type - return YYYY-MM-DD format
+        if isinstance(coltype, Date):
+            return (
+                f"CASE WHEN {value} IS NULL THEN NULL "
+                f"ELSE "
+                f"CAST(DATEPART(YEAR, {value}) AS CHAR(4)) + '-' + "
+                f"RIGHT('0' + CAST(DATEPART(MONTH, {value}) AS VARCHAR(2)), 2) + '-' + "
+                f"RIGHT('0' + CAST(DATEPART(DAY, {value}) AS VARCHAR(2)), 2) "
+                f"END"
+            )
         if isinstance(coltype, Datetime):
             if coltype.precision == 4:
-                return f"CAST({value} AS VARCHAR(100))"
+                return f"CAST({value} AS {varchar_type}(100))"
             if coltype.precision > 0:
                 return (
                     f"CASE WHEN {value} IS NULL THEN NULL "
@@ -227,6 +246,58 @@ class Dialect(BaseDialect):
             return f"CAST({value} AS VARCHAR(100))"
         return f"CAST({value} AS NVARCHAR(100))"
 
+    def timestamp_value(self, t: DbTime) -> str:
+        """Provide SQL for the given timestamp value - match normalize_timestamp precision"""
+        # Use consistent formatting that matches what normalize_timestamp produces
+        # This ensures exact equality comparisons work correctly
+        formatted = t.strftime("%Y-%m-%d %H:%M:%S")
+        if t.microsecond > 0:
+            # Always use 3-digit milliseconds to match normalize_timestamp output
+            # which uses DATEPART(MILLISECOND, value) giving 3 digits
+            milliseconds = t.microsecond // 1000
+            formatted += f".{milliseconds:03d}"
+        return f"'{formatted}'"
+
+    def timestamp_equality_condition(self, column: str, timestamp_value: str) -> str:
+        """Generate a timestamp equality condition that handles precision mismatches"""
+        # For Sybase, we need to handle the case where stored values have microsecond precision
+        # but our query values only have millisecond precision
+
+        # Extract the timestamp without quotes
+        clean_value = timestamp_value.strip("'")
+
+        # If the value has fractional seconds, create a range query
+        if "." in clean_value:
+            # For a value like '2020-01-01 00:02:33.951'
+            # We want to match anything from .951000 to .951999 microseconds
+            base_value = clean_value
+            next_ms_value = self._increment_millisecond(clean_value)
+
+            return f"({column} >= '{base_value}' AND {column} < '{next_ms_value}')"
+        else:
+            # No fractional seconds, use exact match
+            return f"{column} = '{clean_value}'"
+
+    def _increment_millisecond(self, timestamp_str: str) -> str:
+        """Increment the millisecond part of a timestamp string"""
+        from datetime import datetime, timedelta
+
+        try:
+            # Parse the timestamp
+            if "." in timestamp_str:
+                dt = datetime.strptime(timestamp_str, "%Y-%m-%d %H:%M:%S.%f")
+            else:
+                dt = datetime.strptime(timestamp_str, "%Y-%m-%d %H:%M:%S")
+
+            # Add 1 millisecond
+            dt_incremented = dt + timedelta(milliseconds=1)
+
+            # Format back to string with millisecond precision
+            return dt_incremented.strftime("%Y-%m-%d %H:%M:%S.%f")[:-3]
+        except ValueError:
+            # Fallback to original value if parsing fails
+            return timestamp_str
+
     def normalize_number(self, value: str, coltype: FractionalType) -> str:
         return self.to_string(f"CAST({value} AS DECIMAL(38, {coltype.precision}))")
 
@@ -339,6 +410,20 @@ class Dialect(BaseDialect):
             return f"CONVERT(VARCHAR({ch_len}), {value})"
         return f"CONVERT(NVARCHAR({ch_len}), {value})"
 
+    def parse_type(self, table_path: DbPath, info: RawColumnInfo) -> ColType:
+        """Override base parse_type to handle datetime columns that should be treated as dates"""
+
+        # Check if this is a datetime column that should be treated as a date
+        if info.data_type == "datetime":
+            # Sybase IQ stores DATE columns as datetime with precision=4
+            # and DATETIME columns as datetime with precision=8
+            if info.datetime_precision == 4:
+                return Date(
+                    precision=info.datetime_precision,
+                    rounds=self.ROUNDS_ON_PREC_LOSS,
+                )
+        return super().parse_type(table_path, info)
+
     def parse_table_name(self, name: str) -> DbPath:
         "Parse the given table name into a DbPath"
         self.TABLE_NAMES.append(name.split(".")[-1])
